#include <pic.h>

/*************************************************************************\
*          *
*                                                                        *
* 24  1999                                                        *
*                                                                        *
\*************************************************************************/

//=========================================================================
// 
//=========================================================================

#define   FOSC    4000000       //   

#define   FREQ1   100            // 
#define   FREQ2   170            // 
#define   FREQ3   240            //
#define   FREQ4   300            //


#define   J2      RC0           // 
#define   J3      RC1           //
#define   J4      RC2           //

#define   REVERS_JUMPER   J2    //  
#define   F_JUMPER1       J3    //   
#define   F_JUMPER2       J4    //  

#define   STOP_INPUT      RA0   //  /
#define   FAULT_INPUT     RB7   //  Fault 

#define   TIME_FOR_RESTART  FREQ4*6*4 // ~ 4 C

//=========================================================================

typedef unsigned char   byte;
typedef unsigned int    word;
typedef unsigned long   dword;
typedef bit             bool;

#define true  1
#define false 0
#define ON    1
#define OFF   0

byte Index;             //  ()
bool FaultFlag;         // ,     
word Counter;

void NextIndex();       //  
void OutPosition();     //     
void SetFrequency();    //   

word CCP1 @ &CCPR1L;
bank1 byte ADCON1 @ 0x9F;

//=========================================================================
//  
//=========================================================================

void interrupt IntEntry()
{
  if (CCP1IF)
  {
    OutPosition();
    NextIndex();
    SetFrequency();
    CCP1IF=false;
  }
}

//=========================================================================
//    (   )
//=========================================================================

void SetFrequency()
{
  byte temp;

  temp=0;                    //      ,
  if (!F_JUMPER1) temp+=1;   //   temp    0  3
  if (!F_JUMPER2) temp+=2;   //

  switch (temp)
  {                                       //
    case 0:                               //
      CCP1=(FOSC/(4*6)/FREQ1);            // 
      break;                              //  
    case 1:                               // 
      CCP1=(FOSC/(4*6)/FREQ2);            // 
      break;                              // 
    case 2:                               //  
      CCP1=(FOSC/(4*6)/FREQ3);            // /
      break;                              //
    case 3:                               //
      CCP1=(FOSC/(4*6)/FREQ4);            //
      break;                              //
  }                                       //
}

//=========================================================================
//     
//=========================================================================

void OutPosition()
{
  static const byte Out[6]=
  {
    0x55,     // 1 010101B   ; A3    A1    K2         // 
    0x71,     // 1 110001B   ;       A1 K3 K2         //  -  
    0x63,     // 1 100011B   ;    A2 A1 K3            //   
    0x6A,     // 1 101010B   ;    A2    K3    K1      //   
    0x4E,     // 1 001110B   ; A3 A2          K1      //
    0x5C      // 1 011100B   ; A3          K2 K1      //
  };


  if (STOP_INPUT)
  {                   //    ,     
    PORTB=0x3F;       //   ( ),  
    FaultFlag=false;  //    -  ()
  }                 
  else
  {
                                              // 
    if (FaultFlag)                            //  
    {                                         //  
      PORTB=0x3F; if (FAULT_INPUT) Counter--; //
      if (!Counter) FaultFlag=false;          // 
    }                                         //  ,
    else                                      //   
    {                                         // 
      if (FAULT_INPUT==1) PORTB=Out[Index];   // 
      else                                    // 
      {                                       //
        FaultFlag=true; PORTB=0x3F;           //
        Counter=TIME_FOR_RESTART;             //
      }                                       //
    }                                         //
  }                                           //

}

//=========================================================================
//   ()
//=========================================================================

void NextIndex()
{
  if (REVERS_JUMPER)                        //
  {                                         //   
    if (Index==5) Index=0; else Index++;    //  
  }                                         //  
  else                                      //   
  {                                         // 
    if (Index==0) Index=5; else Index--;    //
  }                                         //
}

//=========================================================================
//  
//=========================================================================

void main()
{
  PORTB=0xFF;                   //    
  TRISB=0x80;                   //   -
  PORTB=0x3F;                   //

  TRISC=0xFF;                   //    
  TRISA=0xFF;                   //    A

  ADCON1=0x07;                  //   73 ,    

  Index=0;                      //  
  FaultFlag=false;              //    

  CCP1=0xFFFF;                  //   
  T1CON=0x01;                   //  1      
  CCP1CON=0x0B;                 // 1 ,     1
  CCP1IE=ON;                    //   1   
  CCP1IF=false;                 //    
  PEIE=ON;                      //    
  GIE=ON;                       //   

  while (true) asm("clrwdt");   //    
                                //   
}

//=========================================================================


